@@ -1,7 +1,7 @@
 #############################################################################
-#####################     Easy Debugging Module     #########################
-##################### Copyright 2013 Richard Kelsch #########################
-#####################      All Rights Reserved      #########################
+#################         Easy Debugging Module        ######################
+################# Copyright 2013 - 2015 Richard Kelsch ######################
+#################          All Rights Reserved         ######################
 #############################################################################
 ####### Licensing information available near the end of this file. ##########
 #############################################################################
@@ -14,21 +14,33 @@ use constant {
     FALSE => 0
 };
 
+use DateTime;
 use Term::ANSIColor;
 use Log::Fast;
 use Time::HiRes qw(time);
+use File::Basename;
+use Best [
+ [
+   'Data::Printer' => {'args' => {'colored' => TRUE,'use_prototypes' => FALSE}},
+   'Data::Dumper::Simple',
+   'Data::Dumper'
+ ]
+];
 
 our $Modules = {
-    'Data::Printer'        => 0,
-    'Data::Dumper::Simple' => 0,
-    'Data::Dumper'         => 0,
+    'Data::Printer'        => FALSE,
+    'Data::Dumper::Simple' => FALSE,
+    'Data::Dumper'         => FALSE,
 };
 
+# Now let's set this to whatever module Best loaded.
+$Modules->{Best->which('Data::Printer')} = TRUE;
+
 BEGIN {
     require Exporter;
 
     # set the version for version checking
-    our $VERSION = 0.31;
+    our $VERSION = 0.34;
 
     # Inherit from Exporter to export functions and variables
     our @ISA = qw(Exporter);
@@ -38,31 +50,6 @@ BEGIN {
 
     # Functions and variables which can be optionally exported
     our @EXPORT_OK = qw(@Levels $Modules);
-    # First we try to load in Data::Printer, then Data::Dumper::Simple, else Data::Dumper
-    my $dp = eval {
-        require Data::Printer;
-        Data::Printer->import();
-        1;
-    };
-    if ($dp) {
-        $Modules->{'Data::Printer'} => 1;
-    } else {
-        my $dds = eval {
-            require Data::Dumper::Simple;
-            Data::Dumper::Simple->import();
-            1;
-        };
-        if ($dds) {
-            $Modules->{'Data::Dumper::Simple'} => 1;
-        } else {
-            eval {
-                require Data::Dumper;
-                Data::Dumper->import();
-            };
-        }
-    }
-    
-
 } ## end BEGIN
 
 # Not used, but optionally exported if the coder wants to use it as some sort
@@ -174,7 +161,7 @@ Data::Dumper
 
 END {    # We spit out one last message before we die, the total execute time.
     my $bench = colored(['bright_cyan on_black'], sprintf('%06s', sprintf('%.02f', (time - $MASTERSTART))));
-    $LOG->DEBUG(' %s%s %s', $bench, $ANSILevel{'DEBUG'}, colored(['black on_white'], '---- Script complete ----')) if (defined($ANSILevel{'DEBUG'}) && defined($LOG));
+    $LOG->DEBUG(' %s%s %s', $bench, $ANSILevel{'DEBUG'}, colored(['black on_white'], "---- $0 complete ----")) if (defined($ANSILevel{'DEBUG'}) && defined($LOG));
 }
 
 =head1 METHODS
@@ -259,6 +246,20 @@ output.
 
 =back
 
+=item B<Prefix> [pattern]
+
+A string that is parsed into the output prefix.
+
+DEFAULT:  %D %T %B %L %P
+
+%D = Date (Uses format of "DateStamp" below
+%T = Time (Uses format of "TimeStamp" below
+%B = Benchmark
+%L = Log Level
+%P = Module and subroutine of call
+%d = Just Date (typically used internally only, use %D)
+%t = Just time (typically used internally only, use %T)
+
 =item B<TimeStamp> [pattern]
 
 (See Log::Fast for specifics on these)
@@ -354,7 +355,7 @@ sub new {
     # Log::Fast variables and features can work here.  See the perldocs
     # for Log::Fast for specifics.
     my $class = shift;
-
+    my ($filename,$dir,$suffix) = fileparse($0);
     my $self = {
         'LogLevel'           => 'ERR',                                # Default is errors only
         'Type'               => 'fh',                                 # Default is a filehandle
@@ -369,18 +370,19 @@ sub new {
         'DEBUGMAX_LastStamp' => time,                                 # Initialize the DEBUGMAX benchmark
         'LOG'                => $LOG,                                 # Pull in the global Log::Fast object.
         'Color'              => 1,                                    # Default to colorized output
-        'DateStamp'          => colored(['yellow on_black'], '%D'),
-        'TimeStamp'          => colored(['yellow on_black'], '%T'),
+        'DateStamp'          => colored(['yellow on_black'], '%d'),
+        'TimeStamp'          => colored(['yellow on_black'], '%t'),
         'Padding'            => -25,                                  # Default padding is 25 spaces
-        'Prefix'             => '',
+        'Prefix'             => undef,
+        'Filename'           => '[' . colored(['magenta'],$filename) . ']',
         'ANSILevel'          => {
-            'ERR'       => colored(['white on_red'],       '[ ERROR ]'),
-            'WARN'      => colored(['black on_yellow'],    '[WARNING]'),
-            'NOTICE'    => colored(['black on_magenta'],   '[NOTICE ]'),
-            'INFO'      => colored(['white on_black'],     '[ INFO  ]'),
-            'DEBUG'     => colored(['bold green on_black'],'[ DEBUG ]'),
-            'DEBUGMAX'  => colored(['bold green on_black'],'[-DEBUG-]'),
-            'DEBUGWAIT' => colored(['bold green on_black'],'[*DEBUG*]') # This is highly experimental! Don't whine about it to me, if you use it.
+            'ERR'       => colored(['white on_red'],        '[ ERROR ]'),
+            'WARN'      => colored(['black on_yellow'],     '[WARNING]'),
+            'NOTICE'    => colored(['yellow'],              '[NOTICE ]'),
+            'INFO'      => colored(['black on_white'],      '[ INFO  ]'),
+            'DEBUG'     => colored(['bold green'],          '[ DEBUG ]'),
+            'DEBUGMAX'  => colored(['bold black on_green'], '[-DEBUG-]'),
+            'DEBUGWAIT' => colored(['bold black on_green'], '[*DEBUG*]') # This is highly experimental! Don't whine about it to me, if you use it.
         },
         @_
     };
@@ -403,7 +405,7 @@ sub new {
 
     # This instructs the ANSIColor library to turn off coloring,
     # if the Color attribute is set to zero.
-    $ENV{'ANSI_COLORS_DISABLED'} = 1 if ($self->{'COLOR'} =~ /0|FALSE|OFF/i);
+    $ENV{'ANSI_COLORS_DISABLED'} = TRUE if ($self->{'COLOR'} =~ /0|FALSE|OFF/i);
 
     # If COLOR is FALSE, then clear color data from ANSILEVEL, as these were
     # defined before color was turned off.
@@ -420,8 +422,7 @@ sub new {
 
     # This assembles the Time and Date stamping output.  If any of the
     # variables are blank, then that part will be disabled.
-    $self->{'PREFIX'} = $self->{'DATESTAMP'} . ' ' . $self->{'TIMESTAMP'};
-
+    $self->{'PREFIX'} = '%D %T %B %L[%P] ' unless (defined($self->{'PREFIX'}));
     # The Global loglevel is set here.
     if ($self->{'LOGLEVEL'} eq 'VERBOSE') {
         $self->{'LOG'}->level('INFO');
@@ -435,7 +436,7 @@ sub new {
         {
             'type'   => $self->{'TYPE'},
             'path'   => $self->{'PATH'},
-            'prefix' => $self->{'PREFIX'},
+            'prefix' => '',
             'fh'     => $self->{'FILEHANDLE'}
         }
     );
@@ -465,10 +466,10 @@ The parameters must be passed in the order given
 
 sub debug {
     my $self  = shift;
-    my $level = shift;
+    my $level = uc(shift);
     my $msgs  = shift;
 
-    if ($level !~ /ERR|WARN|NOTICE|INFO|DEBUG/i) { # Compatibility with older versions.
+    if ($level !~ /ERR.*|WARN|NOTICE|INFO|DEBUG/i) { # Compatibility with older versions.
         $level = $msgs;
         $msgs = shift;
     }
@@ -490,25 +491,32 @@ sub debug {
     if (!defined($subroutine) || $subroutine eq '') {    # If there is no subroutine name, then it's usually the "main" namespace
         $subroutine = 'main';
     }
-    if (length($subroutine) > abs($self->{'PADDING'})) {    # Auto adjust padding if it needs to grow
-        $self->{'PADDING'} = 0 - length($subroutine);
-    }
-    my $thisBench = sprintf('%7s', sprintf(' %.02f', time - $self->{$level . '_LASTSTAMP'})) . $self->{'ANSILEVEL'}->{$level} . '[' . colored(['bold cyan on_black'], sprintf('%' . $self->{'PADDING'} . 's>%04d', $subroutine, $cline)) . ']';
+    $cline         = sprintf('%04d',$cline);
+    my $thisBench  = sprintf('%7s', sprintf(' %.02f', time - $self->{$level . '_LASTSTAMP'}));
+    my $thisBench2 = ' ' x length($thisBench);
 
     # For multiline output, only output the bench data on the first line.  Use padded spaces for the rest.
-    my $thisBench2 = ' ' x 7 . $self->{'ANSILEVEL'}->{$level} . '[' . colored(['bold cyan on_black'], sprintf('%' . $self->{'PADDING'} . 's>%04d', $subroutine, $cline)) . ']';
-    my $nested = do {my $n = 0; 1 while caller(2 + $n++); ' ' x $n};    # add padding to nested lines
+    my $nested  = do {my $n = 0; 1 while caller(2 + $n++); ' ' x $n};    # add padding to nested lines
+    $subroutine = $nested . $subroutine;
+    $self->{'PADDING'} = 0 - length($subroutine) if (length($subroutine) >= abs($self->{'PADDING'}));
+    $subroutine = colored(['bold cyan'],sprintf('%' . $self->{'PADDING'} . 's',$subroutine));
     my $first = 1;                                                      # Set the first line flag.
     foreach my $msg (@messages) {                                       # Loop through each line of output and format accordingly.
-        $msg = Dumper($msg) if (ref($msg) ne '');
+        if (ref($msg) ne '') {
+            if ($Modules->{'Data::Printer'}) {
+                $msg = p($msg);
+            } else {
+                $msg = Dumper($msg);
+            }
+        }
         if ($msg =~ /\n/s) {                                            # If the line contains newlines, then it too must be split into multiple lines.
             my @message = split(/\n/, $msg);
             foreach my $line (@message) {                               # Loop through the split lines and format accordingly.
-                $self->_send_to_logger($level, $line, $first, $thisBench, $thisBench2, $nested);
+                $self->_send_to_logger($level, $line, $first, $thisBench, $thisBench2, $subroutine, $cline);
                 $first = 0;                                             # Clear the first line flag.
             }
         } else {    # This line does not contain newlines.  Treat it as a single line.
-            $self->_send_to_logger($level, $msg, $first, $thisBench, $thisBench2, $nested);
+            $self->_send_to_logger($level, $msg, $first, $thisBench, $thisBench2, $subroutine, $cline);
         }
         $first = 0;    # Clear the first line flag.
     }
@@ -522,37 +530,226 @@ sub _send_to_logger {    # This actually simplifies the previous class
     my $first      = shift;
     my $thisBench  = shift;
     my $thisBench2 = shift;
-    my $nested     = shift;
+    my $subroutine = shift;
+    my $cline      = shift;
+
+    $level = 'ERR' if ($level eq 'ERROR'); # This allows the programmer to use either
+    my $dt = DateTime->now();
+    my $Date = $dt->ymd();
+    my $Time = $dt->hms();
+    my $prefix = $self->{'PREFIX'} . ''; # A copy not a pointer
+
+    $prefix =~ s/\%L/$ANSILevel{$level}/g;
+    $prefix =~ s/\%D/$self->{'DATESTAMP'}/g;
+    $prefix =~ s/\%T/$self->{'TIMESTAMP'}/g;
+    $prefix =~ s/\%d/$Date/g;
+    $prefix =~ s/\%t/$Time/g;
+    $prefix =~ s/\%F/$self->{'FILENAME'}/g;
+    if ($first) {
+        $prefix =~ s/\%P/$subroutine/g;
+        $prefix =~ s/\%B/$thisBench/g;
+    } else {
+        $prefix =~ s/\%P/$subroutine /g;
+        $prefix =~ s/\%B/$thisBench2/g;
+    }
 
     if ($level eq 'INFO' && $self->{'LOGLEVEL'} eq 'VERBOSE') {    # Trap verbose flag and temporarily drop the prefix.
-        $self->{'LOG'}->config({'prefix' => ''});
         $self->{'LOG'}->INFO($msg);
-        $self->{'LOG'}->config({'prefix' => $self->{'PREFIX'}});
     } elsif ($level eq 'DEBUGMAX') {                               # Special version of DEBUG.  Outputs as DEBUG in Log::Fast
         if ($self->{'LOGLEVEL'} eq 'DEBUGMAX') {
-            if ($first) {                                          # If the first line, then output benchmark info.
-                $self->{'LOG'}->DEBUG($thisBench . $nested . $msg);
-            } else {                                               # If not the first line, then leave out the benchmark info.
-                $self->{'LOG'}->DEBUG($thisBench2 . $nested . ' ' . $msg);
-            }
+            $self->{'LOG'}->DEBUG($prefix . $msg);
         }
-    } elsif ($level eq 'DEBUGWAIT') {                              # Same as above, but waits for a keypress.  HIGHLY EXPERIMENTAL, no whining
+    } elsif ($level eq 'DEBUGWAIT') {                              # Same as above, but waits for an [ENTER] keypress.  HIGHLY EXPERIMENTAL, no whining
         if ($self->{'LOGLEVEL'} eq 'DEBUGWAIT') {
-            if ($first) {
-                $self->{'LOG'}->DEBUG($thisBench . $nested . $msg);
-            } else {
-                $self->{'LOG'}->DEBUG($thisBench2 . $nested . ' ' . $msg);
-            }
-            <>; # Wait for keypress
+            $self->{'LOG'}->DEBUG($prefix . $msg);
+            <>; # Wait for [ENTER] keypress
         }
     } else {
-        if ($first) {                                              # If the first line, then output benchmark info.
-            $self->{'LOG'}->$level($thisBench . $nested . $msg);
-        } else {                                                   # If not the first line, then leave out the benchmark info.
-            $self->{'LOG'}->$level($thisBench2 . $nested . ' ' . $msg);
-        }
+        $self->{'LOG'}->$level($prefix . $msg);
     }
-} ## end sub send_to_logger
+}
+
+=head2 ERR
+
+This is a friendlier wrapper to 'debug('ERR',['message'])'.  The level is inferred by its name.
+
+=over 1
+
+=item B<MESSAGE>
+
+Either a single string or a reference to a list of strings
+
+=back
+=cut
+
+sub ERR {
+    my $self = shift;
+    $self->debug('ERR',@_);
+}
+
+=head2 ERROR
+
+This is a friendlier wrapper to 'debug('ERR',['message'])'.  The level is inferred by its name.
+
+=over 1
+
+=item B<MESSAGE>
+
+Either a single string or a reference to a list of strings
+
+=back
+=cut
+
+sub ERROR {
+    my $self = shift;
+    $self->debug('ERR',@_);
+}
+
+=head2 WARN
+
+This is a friendlier wrapper to 'debug('WARN',['message'])'.  The level is inferred by its name.
+
+=over 1
+
+=item B<MESSAGE>
+
+Either a single string or a reference to a list of strings
+
+=back
+=cut
+
+sub WARN {
+    my $self = shift;
+    $self->debug('WARN',@_);
+}
+
+=head2 WARNING
+
+This is a friendlier wrapper to 'debug('WARN',['message'])'.  The level is inferred by its name.
+
+=over 1
+
+=item B<MESSAGE>
+
+Either a single string or a reference to a list of strings
+
+=back
+=cut
+
+sub WARNING {
+    my $self = shift;
+    $self->debug('WARN',@_);
+}
+
+=head2 NOTICE
+
+This is a friendlier wrapper to 'debug('NOTICE',['message'])'.  The level is inferred by its name.
+
+=over 1
+
+=item B<MESSAGE>
+
+Either a single string or a reference to a list of strings
+
+=back
+=cut
+
+sub NOTICE {
+    my $self = shift;
+    $self->debug('NOTICE',@_);
+}
+
+=head2 INFO
+
+This is a friendlier wrapper to 'debug('INFO',['message'])'.  The level is inferred by its name.
+
+=over 1
+
+=item B<MESSAGE>
+
+Either a single string or a reference to a list of strings
+
+=back
+=cut
+
+sub INFO {
+    my $self = shift;
+    $self->debug('INFO',@_);
+}
+
+=head2 INFORMATION
+
+This is a friendlier wrapper to 'debug('INFO',['message'])'.  The level is inferred by its name.
+
+=over 1
+
+=item B<MESSAGE>
+
+Either a single string or a reference to a list of strings
+
+=back
+=cut
+
+sub INFORMATION {
+    my $self = shift;
+    $self->debug('INFO',@_);
+}
+
+=head2 DEBUG
+
+This is a friendlier wrapper to 'debug('DEBUG',['message'])'.  The level is inferred by its name.
+
+=over 1
+
+=item B<MESSAGE>
+
+Either a single string or a reference to a list of strings
+
+=back
+=cut
+
+sub DEBUG {
+    my $self = shift;
+    $self->debug('DEBUG',@_);
+}
+
+=head2 DEBUGMAX
+
+This is a friendlier wrapper to 'debug('DEBUGMAX',['message'])'.  The level is inferred by its name.
+
+=over 1
+
+=item B<MESSAGE>
+
+Either a single string or a reference to a list of strings
+
+=back
+=cut
+
+sub DEBUGMAX {
+    my $self = shift;
+    $self->debug('DEBUGMAX',@_);
+}
+
+=head2 DEBUGWAIT
+
+EXPERIMENTAL!!!!  No whining.
+
+This is a friendlier wrapper to 'debug('DEBUGWAIT',['message'])'.  The level is inferred by its name.
+
+=over 1
+
+=item B<MESSAGE>
+
+Either a single string or a reference to a list of strings
+
+=back
+=cut
+
+sub DEBUGWAIT {
+    my $self = shift;
+    $self->debug('DEBUGWAIT',@_);
+}
 
 =head1 INSTALLATION
 
@@ -581,7 +778,7 @@ modify it under the same terms as Perl itself.
 
 =head1 VERSION
 
-Version 0.31    (April 06, 2015)
+Version 0.34    (June 01, 2015)
 
 =head1 BUGS
 
